/*
  A Dummy server to allow the STARTSServer application and the freeWAIS
  server to run on different machines.  Refer to the STARTS implementation
  notes for more information.
  */


// This example is from the book _Java in a Nutshell_ by David Flanagan.
// Written by David Flanagan.  Copyright (c) 1996 O'Reilly & Associates.
// You may study, use, modify, and distribute this example for any purpose.
// This example is provided WITHOUT WARRANTY either expressed or implied.

import java.io.*;
import java.net.*;

import wais.*;
import resource.*;

public class DummyServer extends Thread {
    public final static int DEFAULT_PORT = 6790;
    protected int port;
    protected ServerSocket listen_socket;

    // Exit with an error message, when an exception occurs.
    public static void fail(Exception e, String msg) {
        System.err.println(msg + ": " +  e);
        System.exit(1);
    }

    // Create a ServerSocket to listen for connections on;  start the thread.
    public DummyServer(int port) {
        if (port == 0) port = DEFAULT_PORT;
        this.port = port;
        try { listen_socket = new ServerSocket(port); }
        catch (IOException e) { fail(e, "Exception creating server socket"); }
        System.out.println("Server: listening on port " + port);
        this.start();
    }

    // The body of the server thread.  Loop forever, listening for and
    // accepting connections from clients.  For each connection,
    // create a Connection object to handle communication through the
    // new Socket.
    public void run() {
        try {
            while(true) {
                Socket client_socket = listen_socket.accept();
                DummyConnection c = new DummyConnection(client_socket);
            }
        }
        catch (IOException e) {
            fail(e, "Exception while listening for connections");
        }
    }

    // Start the server up, listening on an optionally specified port
    public static void main(String[] args) {
        int port = 0;
        if (args.length == 1) {
            try { port = Integer.parseInt(args[0]);  }
            catch (NumberFormatException e) { port = 0; }
        }
        new DummyServer(port);
    }
}

// This class is the thread that handles all communication with a client
class DummyConnection extends Thread {
    protected Socket client;
    protected DataInputStream in;
    protected PrintStream out;

    // Initialize the streams and start the thread
    public DummyConnection(Socket client_socket) {
        client = client_socket;
        try {
	  System.out.println("Got a connection");
            in = new DataInputStream(client.getInputStream());
            out = new PrintStream(client.getOutputStream());
        }
        catch (IOException e) {
            try { client.close(); } catch (IOException e2) { ; }
            System.err.println("Exception while getting socket streams: " + e);
            return;
        }
        this.start();
    }

    // Provide the service.
    // Read a line, reverse it, send it back.
    public void run() {
        String query, source;
        StringBuffer revline;
        int len;
	String[] results;
        try {
            for(;;) {
                // read in a line
                query = in.readLine();
                if (query == null) break;
		source = in.readLine();
		if (source == null) break;

		System.out.println("Query is: " + query);
		System.out.println("Source is: " + source);
		results = WAISSearchSend.SearchWAIS("cs-tr.cs.cornell.edu", "5000", source, query);
		int numResults = 0;
		if (results != null) {
		  numResults = results.length;
		}
		out.println(String.valueOf(numResults));
		for (int i = 0; i < numResults; i++) {
		  out.println(results[i]);
		}
            }
        }
        catch (IOException e) { ; }
	catch (SourceException e) {
	  System.out.println(e.getMessage());
	}
        finally { try {client.close();} catch (IOException e2) {;} }
    }
}
