/**********************************************************************
 *                              STARTS                                *
 *      Stanford Protocol for Internet Retrieval and Search           *
 *                                                                    *
 *                      Reference Implementation                      *
 *                                                                    *
 *                           Carl Lagoze                              *
 *                      lagoze@cs.cornell.edu                         *
 *                   Department of Computer Science                   *
 *                        Cornell University                          *
 *********************************************************************/

package STARTSConfiguration;

import java.util.*;

import resource.*;
import soif.*;

/**
 * The descripton of the resource that this STARTS server provides
 * services for.
 */
public final class ResourceDescription{

     /**
      * Hashtable to map from a source name to its description.
      */
     static Hashtable sources = new Hashtable();

     // Load the sources hashtable
     static {
        sources.put("cstr", new CSTRSourceDescription());
        sources.put("linux", new LINUXSourceDescription());
     }

    /**
     * Return the SourceDescription that maps to a particular
     * source name.
     * @param sourceName   name of the source to return a
     * SourceDescription for.
     * @return the description object for the requested source.
     */
     public SourceDescription GetSource(String sourceName) {
        return((SourceDescription) sources.get(sourceName));
     }

     /**
         Return the SOIFObject that is the resource metadata for this
         resource.
         @return a SOIFObject describing the resource.
     */
     public SOIFObject ResourceToSOIF() {
        SOIFObject outSOIF = new SOIFObject("SResource");

        outSOIF.SetAttributeValue("Version",
           ServerConfiguration.STARTSVersion);

        StringBuffer buf = new StringBuffer();
        for (Enumeration e = sources.keys(); e.hasMoreElements();) {
            String key = (String) e.nextElement();
            SourceDescription source = (SourceDescription) sources.get(key);
            buf.append(source.GetSourceID() +
                " " +
                source.GetMetadataURL() +
                " " +
                source.GetMetadataSyntax() + "\n");
        }
        outSOIF.SetAttributeValue("SourceList", buf.toString());

        return outSOIF;
    }

}

