/**********************************************************************
 *                              STARTS                                *
 *      Stanford Protocol for Internet Retrieval and Search           *
 *                                                                    *
 *                      Reference Implementation                      *
 *                                                                    *
 *                           Carl Lagoze                              *
 *                      lagoze@cs.cornell.edu                         *
 *                   Department of Computer Science                   *
 *                        Cornell University                          *
 *********************************************************************/

package STARTSConfiguration;

/**
    General configuration information for the STARTS implementation
*/
public final class ServerConfiguration {

    public static String STARTSVersion = "STARTS 1.1";

    // query defaults

    /**
        default value for dropping stop words
    */
    public static boolean dropStopWords  = true;

    /**
        default attribute set
    */
    public static String defaultAttributeSet = "basic-1";

    /**
        default answer fields
    */
    public static String defaultAnswerFields[] = {"title", "author"};

    /**
        default sorting key
    */
    public static String defaultSortFields[] = {"Score"};

    /**
        ascending or descending on default sort key
    */
    public static boolean sortDescending = true;

    /**
        default minimum document score.
    */
    public static float minimumDocScore = Float.MIN_VALUE;

    /**
        default maximum number of documents returned.
    */
    public static int maximumNumDocuments = 20;

    /**
        host machine on which the application is running
    */
    public static String hostName = "cs-tr.cs.cornell.edu";
}