/**********************************************************************
 *                              STARTS                                *
 *      Stanford Protocol for Internet Retrieval and Search           *
 *                                                                    *
 *                      Reference Implementation                      *
 *                                                                    *
 *                           Carl Lagoze                              *
 *                      lagoze@cs.cornell.edu                         *
 *                   Department of Computer Science                   *
 *                        Cornell University                          *
 *********************************************************************/

package http;

import java.util.*;

/**
 * A Class to handle HTTP-specific functions
 */

public class HTTPHeader {

  /*
   * The versioin of HTTP that we support.
   */
    static String HTTPVersion = "HTTP/1.0";

  /*
   * Hashtable to hold HTTP code messages.
   */
    static Hashtable HTTPCodes = new Hashtable();

  // Load the HTTPCodes table
    static {
        HTTPCodes.put(new Integer(200),"OK");
        HTTPCodes.put(new Integer(201),"Created");
        HTTPCodes.put(new Integer(400),"Bad request");
        HTTPCodes.put(new Integer(401),"Unauthorized");
        HTTPCodes.put(new Integer(402),"Payment required");
        HTTPCodes.put(new Integer(403),"Forbidden");
        HTTPCodes.put(new Integer(404),"Not found");
        HTTPCodes.put(new Integer(500),"Internal error");
        HTTPCodes.put(new Integer(501),"Not implemented");
        HTTPCodes.put(new Integer(301),"Moved");
        HTTPCodes.put(new Integer(302),"Temporary relocation URL follows");
        HTTPCodes.put(new Integer(303),"Method (use a different method)");
    }

  /**
   * Construct a HTTP header
   * @param code Return code of the HTTP response.
   * @param contentType MIME content type of the response.
   * @return a String containing an HTTPD header.
   */
    public static String MakeHeader(int code, String contentType) {
        StringBuffer buf = new StringBuffer();
        String ResponseMessage =
            (String) HTTPCodes.get(new Integer(code));

        buf.append(HTTPVersion + " " + String.valueOf(code) +
            " " + ResponseMessage + "\n");
        buf.append("MIME-version: 1.0\n");
        buf.append("Content-type: " + contentType + "\n");
        buf.append("\n");
        return buf.toString();
    }
}
