/* parse syntax for STARTS filter expression */
%{
import query.*;
import java.util.*;
%}

%token BOOLEANOP
%token PROX
%token INTEGER_LITERAL
%token BOOLEANVAL
%token QUOTED_STRING
%token LANGUAGE_CODE
%token STRING_TEXT
%token UNQUOTED_STRING
%token RELATION
%token MODIFIER

/* %type <ExpNode> filter
%type <ExpNode> prox_op
%type <ExpNode> term
%type <ExpNode> lstring
%type <ExpNode> field
%type <ExpNode> modifier_list
%type <ExpNode> modifier
%type <ExpNode> attribute_set
%type <ExpNode> field_spec*/

/* Grammar follows */
%%
filter:	term
	| '(' term prox_op term ')'
	{
		((ExpNode) $3).AddNewChild((ExpNode)$2);
		((ExpNode) $3).AddNewChild((ExpNode)$4);
		$$ = $3;
	}
	| '(' filter booleanop filter ')'
	{
		((ExpNode) $3).AddNewChild((ExpNode)$2);
		((ExpNode) $3).AddNewChild((ExpNode)$4);
		$$ = $3;
	}
	;

booleanop: BOOLEANOP
	{
		$$ = new BoolOp(query,yytext.toString());
	}
	;

prox_op: PROX '[' digit booleanval ']'
	{
		$$ = new ProxOp(query,Integer.getInteger((String) $3).intValue(),
				((String)$4).equals("T"));
	}
	;

digit: INTEGER_LITERAL
	{
		$$ = yytext.toString();
	}
	;

booleanval: BOOLEANVAL
	{
		$$ = yytext.toString();
	}
	;

term:	lstring
	{
		$$ = new Term(query,(LString) $1);
	}
	| '(' field modifier_list lstring ')'
	{
		$$ = new Term(query,(Field) $2, (ModifierList) $3, (LString) $4);
	}
	| '(' modifier_list lstring ')'
	{
		$$ = new Term(query,(ModifierList) $2, (LString) $3);
	}
	| '(' field lstring ')'
	{
		$$ = new Term(query,(Field) $2, (LString) $3);
	}
	;

lstring: quoted_string
	{
		$$ = new LString(query,yytext.toString());
	}
	| '[' language_code quoted_string ']'
	{
		$$ = new LString(query,(String) $2, (String) $3)
	}
	;

field: field_spec
	{
		$$ = new Field(query,(String) $1);
	}
	| '[' attribute_set field_spec ']'

	{
		$$ = new Field(query,(String) $2,(String) $3);
	}
	;

modifier_list:  modifier
	{
	  	$$ = new ModifierList(query);
		((ModifierList)$$).AddNewModifier((Modifier) $1);

	}
	| modifier modifier_list
	{
	  	((ModifierList) $2).AddNewModifier((Modifier)$1);
		$$ = $2;
	}
	;

modifier: RELATION
	{
		$$ = new Modifier(query,yytext.toString());
	}
	| MODIFIER
	{
		$$ = new Modifier(query,yytext.toString());
	}
	| '{' attribute_set field_spec '}'
	{
		$$ = new Modifier(query,(String) $2, (String) $3);
	}
	;

attribute_set: UNQUOTED_STRING
	{
		$$ = yytext.toString();
	}
	;

field_spec: UNQUOTED_STRING
	{
		$$ = yytext.toString();
	}
        ;

language_code: LANGUAGE_CODE
	{
		$$ = yytext.toString();
	}
	;

quoted_string: QUOTED_STRING
	{
		$$ = yytext.toString();
	}
	;

%%

Query query;

public void SetQuery(Query query) {
    this.query = query;
}

/* Override yyerror */
public
void
yyverror(String s) throws ParseException
{
    String msg = "Error parsing filter expression: " + yyerror_verbose(s);
    yyerror(msg);
}

