%{
import YYtokentypes;
%}

ALPHANUMCHAR [A-Za-z0-9]
DIGIT [0-9]
WHITESPACE [\n\ \t\b\r\f]
STRING_TEXT (\\\"|[^\n\"]|\\{WHITESPACE}+\\)*

%%

<INITIAL>{
"and"			{yyreturn(BOOLEANOP);}
"or"			{yyreturn(BOOLEANOP);}
"and-not"		{yyreturn(BOOLEANOP);}

"("			{yyreturn('(');}
")"			{yyreturn(')');}
"["			{yyreturn('[');}
"]"			{yyreturn(']');}
"{"			{yyreturn('{');}
"}"			{yyreturn('}');}

"<" 			{yyreturn(RELATION);}
"<="			{yyreturn(RELATION);}
"="			{yyreturn(RELATION);}
">="			{yyreturn(RELATION);}
">"			{yyreturn(RELATION);}
"!="			{yyreturn(RELATION);}

"phonetic"		{yyreturn(MODIFIER);}
"stem"			{yyreturn(MODIFIER);}
"thesaurus"		{yyreturn(MODIFIER);}
"right-truncation"	{yyreturn(MODIFIER);}
"left-truncation"	{yyreturn(MODIFIER);}
"case-sensitive"	{yyreturn(MODIFIER);}

"prox"			{yyreturn(PROX);}
"T"			{yyreturn(BOOLEANVAL);}
"F"			{yyreturn(BOOLEANVAL);}


"list"			{yyreturn(LIST);}


{DIGIT}+	{ /* integer */
  		yyreturn(INTEGER_LITERAL);
	}

\.{DIGIT}	{ /* decimal value between 0 and 1 */
		yyreturn(DECIMAL_LITERAL);
	}

\"{STRING_TEXT}\" { /* string in quotes*/
		yyreturn(QUOTED_STRING);
		}

\"{STRING_TEXT} { yylexerror("unclosed string");}

[A-Z]{1,8}(-[A-Z]{1,8})+  {/* rfc1766 language encoding */
		yyreturn(LANGUAGE_CODE);
		}

[A-Za-z0-9\-_]+	{ /* unquoted string */
		yyreturn(UNQUOTED_STRING);
		}

}

%%

// Include the set of tokentypes to
// avoid having to say e.g. Tokentypes.INTEGER_LITERAL

// TOKENTYPES

@TOKENTYPES@

protected static final char LBRACE = '{';
protected static final char RBRACE = '}';
