/* parse syntax for STARTS ranking expression */
%{
import query.*;
import java.util.*;
%}

%token BOOLEANOP
%token PROX
%token INTEGER_LITERAL
%token DECIMAL_LITERAL
%token BOOLEANVAL
%token QUOTED_STRING
%token LANGUAGE_CODE
%token UNQUOTED_STRING
%token RELATION
%token MODIFIER
%token LIST


/* Grammar follows */
%%

ranking: 	term
	{
		$$ = $1;
	}
	| '(' term prox_op term ')'
	{
		((ExpNode) $3).AddNewChild((ExpNode)$2);
		((ExpNode) $3).AddNewChild((ExpNode)$4);
		$$ = $3;
	}
	| LIST '(' ranking_list ')'
	{
		$$ = $3;
	}
	| LIST '(' w_ranking_list ')'
	{
		$$ = $3;
	}
	| '(' ranking booleanop ranking ')'
	{
		((ExpNode) $3).AddNewChild((ExpNode)$2);
		((ExpNode) $3).AddNewChild((ExpNode)$4);
		$$ = $3;
	}
	| '(' w_ranking booleanop w_ranking ')'
	{
		((ExpNode) $3).AddNewChild((ExpNode)$2);
		((ExpNode) $3).AddNewChild((ExpNode)$4);
		$$ = $3;
	}
	;

ranking_list: ranking
	{
		$$ = new RankingList(query);
		((ExpNode) $$).AddNewChild((ExpNode) $1);
	}
	| ranking ranking_list
	{
		((ExpNode) $2).AddNewChild((ExpNode) $1);
		$$ = $2;
	}
	;

w_ranking_list: w_ranking
	{
		$$ = new RankingList(query);
		((ExpNode) $$).AddNewChild((ExpNode) $1);
	}
	| w_ranking w_ranking_list
	{
		((ExpNode) $2).AddNewChild((ExpNode) $1);
		$$ = $2;
	}
	;

w_ranking: '(' ranking decimal_digit ')'
	{
		$$ = new WeightedRanking
			(query, Float.valueOf((String) $3).floatValue());
		((ExpNode) $$).AddNewChild((ExpNode) $2);
	}
	;


booleanop: BOOLEANOP
	{
		$$ = new BoolOp(query,yytext.toString());
	}
	;

prox_op: PROX '[' digit booleanval ']'
	{
		$$ = new ProxOp(query,Integer.getInteger((String) $3).intValue(),
				((String)$4).equals("T"));
	}
	;

digit: INTEGER_LITERAL
	{
		$$ = yytext.toString();
	}
	;

decimal_digit: DECIMAL_LITERAL
	{
		$$ = yytext.toString();
	}
	;

booleanval: BOOLEANVAL
	{
		$$ = yytext.toString();
	}
	;

term:	lstring
	{
		$$ = new Term(query,(LString) $1);
	}
	| '(' field modifier_list lstring ')'
	{
		$$ = new Term(query,(Field) $2, (ModifierList) $3, (LString) $4);
	}
	| '(' modifier_list lstring ')'
	{
		$$ = new Term(query,(ModifierList) $2, (LString) $3);
	}
	| '(' field lstring ')'
	{
		$$ = new Term(query,(Field) $2, (LString) $3);
	}
	;

lstring: quoted_string
	{
		$$ = new LString(query,yytext.toString());
	}
	| '[' language_code quoted_string ']'
	{
		$$ = new LString(query,(String) $2, (String) $3)
	}
	;

field: field_spec
	{
		$$ = new Field(query,(String) $1);
	}
	| '[' attribute_set field_spec ']'

	{
		$$ = new Field(query,(String) $2,(String) $3);
	}
	;

modifier_list:  modifier
	{
	  	$$ = new ModifierList(query);
		((ModifierList)$$).AddNewModifier((Modifier) $1);

	}
	| modifier modifier_list
	{
		((ModifierList) $2).AddNewModifier((Modifier) $1);
		$$ = $2;
	}
	;

modifier: RELATION
	{
		$$ = new Modifier(query,yytext.toString());
	}
	| MODIFIER
	{
		$$ = new Modifier(query,yytext.toString());
	}
	| '{' attribute_set field_spec '}'
	{
		$$ = new Modifier(query,(String) $2, (String) $3);
	}
	;

attribute_set: UNQUOTED_STRING
	{
		$$ = yytext.toString();
	}
	;

field_spec: UNQUOTED_STRING
	{
		$$ = yytext.toString();
	}

language_code: LANGUAGE_CODE
	{
		$$ = yytext.toString();
	}
	;

quoted_string: QUOTED_STRING
	{
		$$ = yytext.toString();
	}
	;

%%
Query query;

public void SetQuery(Query query) {
    this.query = query;
}

/* Override yyerror */
public
void
yyverror(String s) throws ParseException
{
    String msg = "Error parsing ranking expression: " + yyerror_verbose(s);
    yyerror(msg);
}

