/**********************************************************************
 *                              STARTS                                *
 *      Stanford Protocol for Internet Retrieval and Search           *
 *                                                                    *
 *                      Reference Implementation                      *
 *                                                                    *
 *                           Carl Lagoze                              *
 *                      lagoze@cs.cornell.edu                         *
 *                   Department of Computer Science                   *
 *                        Cornell University                          *
 *********************************************************************/
/**
 * Class added by Sandy Payette 6/23/97
 */

package query;

import java.util.*;


/**
    Abstract superclass for all attribute sets which STARTS can
    understand and operate on.
**/

public abstract class AttrSet {

// characteristics of an attribute set object


    // Load the attribute set field table (attrSetFieldTable) which
    // contains all possible fields in the attribute set.

    protected abstract void LoadFieldTable();

    /**
     * predicate to determine whether an attribute set supports
     * a particular field.
     * @param field    field to test
     * @return true or false depending on if the field is supported
     * by the attribute set.
     */

	protected abstract boolean attrSetFieldSupported_p(String field);


    // Load source-specific fields hashtable which contains only the
    // attribute set fields supported by the source

    protected abstract void LoadSourceFields(String [][] fieldsTranslation);

    /**
     * Predicate to determine whether a specific field/attribute set
     * pair is supported by the source.
     * @param field    field to test
     * @return true of false depending on if the field/attribute set
     * pair is supported at the source.
     */

	public abstract boolean FieldSupportBySource(String field);




    // Load Hashtable for identification of source-specific fields and for
    // translation of source-specific fields

    protected abstract void LoadFieldXlate(String [][] fieldsTranslation);


    /**
     * Return a String that is a translation from STARTS field
     * to a WAIS field.
     * @param field    Field name of the source field to be translated.
     * @return the String translation of the input field.
     */

    public abstract String TranslateField(String field);


}