/**********************************************************************
 *                              STARTS                                *
 *      Stanford Protocol for Internet Retrieval and Search           *
 *                                                                    *
 *                      Reference Implementation                      *
 *                                                                    *
 *                           Carl Lagoze                              *
 *                      lagoze@cs.cornell.edu                         *
 *                   Department of Computer Science                   *
 *                        Cornell University                          *
 *********************************************************************/

package query;

import resource.*;

/**
 * A STARTS modifier.
 */
public class Modifier
{

    /**
     * the string that is the modifier
     */
	String modifier;

    /**
     * the attribute set of the modifier.
     */
	String attributeSet;

    /**
     * the query context of the modifier
     */
	Query query;

	/**
	 * construct with default  attribute set
	 * @param query    query context of this modifier
	 * @param modifier the string that is the modifier
	 */
	public Modifier(Query query, String modifier) {
	    this.query = query;
		this.modifier = modifier;
		this.attributeSet = query.defaultAttributeSet;
	}

	/**
	 * construct a modifier with a specified attribute set
	 * @param query    query context of this modifier
	 * @param attributeSet attribute set of this modifier
	 * @param modifier the string that is the modifier
	 */
	public Modifier(Query query,String attributeSet, String modifier) {
        this.query = query;
		this.modifier = modifier;
		this.attributeSet = attributeSet;
	}

    /**
     * return the string that is the modifier.
     * @return the string that is the modifier (e.g. soundex)
     */
	public String GetModifier() {
		return(modifier);
	}

    /**
     * return the attribute set of this modifier
     * @return the string that is the attribute set of this modifier
     * (e.g., basic-1)
     */
	public String GetAttributeSet() {
		return(attributeSet);
	}

    /**
     * predicate returning whether this modifier is supported in the
     * source.
     * @return true or false depening on whether this modifier is suported
     * for the source in the query context.
     */
    public boolean Supported_p() {
        return(query.source.ModifierSupported_p(
            GetAttributeSet(), GetModifier()));
    }

    /**
     * predicate to determine whether a language is supported for
     * this modifier
     * @param language the language to check.
     * @return True or false depending if the input language is supported
     * for this modifier.
     */
	public boolean LanguageSupported_p(String language)  {
	    return(query.source.LanguageSupportedForModifier_p(
	        GetAttributeSet(), GetModifier(), language));
	}


}

