/**********************************************************************
 *                              STARTS                                *
 *      Stanford Protocol for Internet Retrieval and Search           *
 *                                                                    *
 *                      Reference Implementation                      *
 *                                                                    *
 *                           Carl Lagoze                              *
 *                      lagoze@cs.cornell.edu                         *
 *                   Department of Computer Science                   *
 *                        Cornell University                          *
 *********************************************************************/

package query;

import java.util.*;

/**
 * Interface to define the methods common to all nodes that appear
 * in ranking expressions.
 */
public interface RankingNode {

    /**
     * Return the sub-tree that represents the source specific
     * translation of the sub-tree rooted at this Ranking node
     * @return the subtree that is the source equivalent of this ranking
     * sub-tree.
     */
    public ExpNode TranslateToRanking() throws SourceQueryException;

    /**
     * Extract the terms in the ranking sub-tree rooted by this node
     * @param termVector   the vector of terms in the sub-tree
     */
    public void ExtractTerms(Vector termVector);
}
