/**********************************************************************
 *                              STARTS                                *
 *      Stanford Protocol for Internet Retrieval and Search           *
 *                                                                    *
 *                      Reference Implementation                      *
 *                                                                    *
 *                           Carl Lagoze                              *
 *                      lagoze@cs.cornell.edu                         *
 *                   Department of Computer Science                   *
 *                        Cornell University                          *
 *********************************************************************/

package results;

import java.lang.*;

/**
 * The statistics collected for terms within documents that are
 * part of a STARTS results set.  An instance of a TermStatistics
 * is always attached to an instance of a Document.
 */
public class TermStatistics {

    /**
     * The field of the term
     */
    String field;

    /**
     * The string that is the term itself.
     */
    String term;

    /**
     * The number of times that this term appears in the document with
     * which this term statistics is related.
     */
    long frequency;

    /**
     * The weight of the query term in the document.
     */
    float weight;

    /**
     * The number of documents in the source that contain this term.
     */
    long documentFrequency;

    /**
     * Construct a new term statistics
     * @param  field   The field of the term
     * @param  term    The string that is the term itself.
     * @param frequency    The number of times that this term
     * appears in the document.
     * @param weight   The weight of the query term in the document.
     * @param documentFrequency    The number of documents in the
     * source that contain this term.
     */
    public TermStatistics(String field, String term,
        long frequency, float weight, long documentFrequency) {

        this.field = field;
        this.term = term;
        this.frequency = frequency;
        this.weight = weight;
        this.documentFrequency = documentFrequency;
    }

    /**
     * Return a string represenation of this TermStats instance
     * @return the string representation of the statistics for this term.
     */
    public String GenerateStatsString() {
        return("(" + this.field + " \"" + term + "\"" + ")" + " " +
            String.valueOf(frequency) + " " + String.valueOf(weight) + " " +
            String.valueOf(documentFrequency));
    }
}
