/**********************************************************************
 *                              STARTS                                *
 *      Stanford Protocol for Internet Retrieval and Search           *
 *                                                                    *
 *                      Reference Implementation                      *
 *                                                                    *
 *                           Carl Lagoze                              *
 *                      lagoze@cs.cornell.edu                         *
 *                   Department of Computer Science                   *
 *                        Cornell University                          *
 *********************************************************************/

package wais;

import java.util.*;

import query.*;
import results.*;

/**
 * An ExpNode that represents a Boolean Operator in a WAIS query
 * expression.  Each WAISBoolOp has two children, which are the
 * operands of the WAISBoolOp and one property:
 * operator - The operator (e.g., and, or, etc.)
 */
public class WAISBoolOp extends ExpNode implements resource.SourceNode{

    /**
     * Construct a new WAISBoolOp
     * @param query    The query context of this WAISBoolOp
     * @param operator The operator (e.g., and , or)
     */
    public WAISBoolOp(Query query, String operator) {
        super(query);
        SetPropertyValue("operator", operator);
    }

    /**
     * Return the string that is the operator for this WAISBoolOp
     */
    public String GetOperator() {
	    return((String) GetPropertyValue("operator"));
    }

    /**
     * Return the string that represents the unparsed representation
     * of the sub-tree rooted at this WAISBoolOp.
     */
    public String Unparse() {
        Vector children;
        String child0String, child1String;

        children = GetChildren();
        child0String =
            "(" + ((ExpNode) children.elementAt(0)).Unparse() + ")";
        child1String =
            "(" + ((ExpNode) children.elementAt(1)).Unparse() + ")";

        return(child0String + " " + GetOperator() + " " + child1String);
    }

    /**
     * This is a slot to do some primitive ranking on this sub-tree.
     * Unimplemented now except for the recursion
     * @param doc  The document to rank using this sub-tree
     */
    public float RankBy(Document doc) {
        Vector children = GetChildren();
        float lRank = ((resource.SourceNode) children.elementAt(0)).RankBy(doc);
        float rRank = ((resource.SourceNode) children.elementAt(1)).RankBy(doc);
        return ((float) 0.0);
    }

}
