/**********************************************************************
 *                              STARTS                                *
 *      Stanford Protocol for Internet Retrieval and Search           *
 *                                                                    *
 *                      Reference Implementation                      *
 *                                                                    *
 *                           Carl Lagoze                              *
 *                      lagoze@cs.cornell.edu                         *
 *                   Department of Computer Science                   *
 *                        Cornell University                          *
 *********************************************************************/

package wais;

/**
 * Class to represent the information held in an entry in the WAIS dictionary file.  The
 * three relevant pieces of information are:
 * - the string that is the term
 * - the location in the inverted index file of this term
 * - the number of documents in the source that contain this term.
 */
public class WAISDictionaryTerm {

    /**
     * The term for this instance.
     */
    String term;

    /**
     * The offset in the inverted index file of this term.
     */
    long invertedFileLocation;

    /**
     * The number of documents in the source that this term appears
     * in.
     */
    int numberDocuments;

    /**
     * Construct a new WAISDictionaryTerm
     * @param term The string that is the term.
     * @param loc  The location in the inverted index file of this term
     * @param number   The number of documents in the source that
     * contain this term.
     */
    public WAISDictionaryTerm(String term, long loc, int number) {
        this.term = term;
        this.invertedFileLocation = loc;
        this.numberDocuments = number;
    }

    /**
     * Return the term associated with this instance.
     */
    public String GetTerm() {
        return term;
    }

    /**
     * Return the inverted index file offset of this term.
     */
    public long GetInvertedFileLocation() {
        return invertedFileLocation;
    }

    /**
     * Return the number of documents in the source that this term
     * appears in.
     */
    public int GetNumberDocuments() {
        return numberDocuments;
    }
}
