/**********************************************************************
 *                              STARTS                                *
 *      Stanford Protocol for Internet Retrieval and Search           *
 *                                                                    *
 *                      Reference Implementation                      *
 *                                                                    *
 *                           Carl Lagoze                              *
 *                      lagoze@cs.cornell.edu                         *
 *                   Department of Computer Science                   *
 *                        Cornell University                          *
 *********************************************************************/

package wais;

/**
 * Container for document specific information for WAIS terms.  Each
 * instance of a WAISTermDocInfo is associated with a document in
 * a result set.
 */
public class WAISTermDocInfo {

    /**
     * The weight of the term with respect to the document.
     */
    float weight;

    /**
     * The number of occurrences of the term in the document.
     */
    int termFrequency;

    /**
     * Construct a new WAISTermDocInfo
     * @param weight   Weight of the term with respect to the
     * document.
     * @param termFrequency    The number of occurrences of the term
     * in the document.
     */
    public WAISTermDocInfo(float weight, int termFrequency) {
        this.weight = weight;
        this.termFrequency = termFrequency;
    }

    /**
     * return the weight of the term with respect to the document.
     */
    public float GetWeight() {
        return(weight);
    }

    /**
     * return the number of occurrences of the term in the document.
     */
    public int GetTermFrequency() {
        return(termFrequency);
    }
}
