/**********************************************************************
 *                              STARTS                                *
 *      Stanford Protocol for Internet Retrieval and Search           *
 *                                                                    *
 *                      Reference Implementation                      *
 *                                                                    *
 *                           Carl Lagoze                              *
 *                      lagoze@cs.cornell.edu                         *
 *                   Department of Computer Science                   *
 *                        Cornell University                          *
 *********************************************************************/


/*
  This is a modified version of the waissearch program distributed with
  freeWAIS.  Modifications include:
  - changed from a main program to a subroutine call.
  - added java type translations so this can be called form a java
  native method.
  - changed error handling to exception raising.
  - return the source ID (pathname) with each hit.
  */

/* CJL added this to get rid of compiler conflicts */
#define __BIT_TYPES_DEFINED__

/* WIDE AREA INFORMATION SERVER SOFTWARE:
   No guarantees or restrictions.  See the readme file for the full standard
   disclaimer.

   Brewster@think.com
*/

/* Copyright (c) CNIDR (see ../doc/CNIDR/COPYRIGHT) */

/* Modified version of waissearch.c by Carl Lagoze to provide link to 
   java STARTS implementation
   */


#include <ui.h>
#include <sockets.h>
#include <docid.h>
/* Security patch from mike@tab00.larc.nasa.gov */
#include <cdialect.h>
/*#include <sys/types.h>*/

/* CJL - includes added for java links */
#include "wais_WAISSearchSend.h"
#include <javaString.h>

#define MAX_MESSAGE_LEN BUFSZ
#define CHARS_PER_PAGE 10000 /* number of chars retrieved in each request */
#define MAX_FILE_NAME_LEN 1000
#define Z39_50_SERVICE "210"

#define WAISSEARCH_DATE "Thu Aug 27 1992"

#define RESP_BUF_LEN 256
#define ERR_BUF_LEN 256

HArrayOfObject *alloc_class_array(char *, int);

/* throw a java exception */
void throw_e(msg)
char *msg;
{
  SignalError(EE(), "SourceException", msg);
}

HArrayOfObject *alloc_class_array (char *type, int cnt) 
  {
    HArrayOfObject *retval;

    retval = (HArrayOfObject *)ArrayAlloc(T_CLASS, cnt);
    unhand(retval)->body[cnt] =
      (HObject *)FindClass(EE(), type, TRUE);
    return retval;
  }

/* modified from Jonny G's version in ui/question.c */
void showDiags(d)
diagnosticRecord **d;
{
  long i;
  char *err_buf;

  for (i = 0; d[i] != NULL; i++) {
    if (d[i]->ADDINFO != NULL) {
      err_buf = (char *)
	  s_malloc((size_t)ERR_BUF_LEN * sizeof(char));
      sprintf(err_buf, "Code: %s, %s\n", d[i]->DIAG, d[i] ->ADDINFO);
      throw_e(err_buf);
    }
  }
}

/* modified from tracy shen's version in wutil.c
 * displays either a text record of a set of headlines.
 */
HArrayOfString
*display_search_response(response)
SearchResponseAPDU *response;

{
  WAISSearchResponse  *info;
  long i, k;
  char **responses;
  char *err_buf;
  HArrayOfString *retval = NULL;
  ClassArrayOfString *retstrs;
  


  retval = (HArrayOfString *) alloc_class_array("/java/lang/String", 
			      response->NumberOfRecordsReturned);
  retstrs = unhand(retval);
  responses = (char **) 
    s_malloc((size_t)response->NumberOfRecordsReturned * sizeof(char *));
  if ( response->DatabaseDiagnosticRecords != 0 ) {
    info = (WAISSearchResponse *)response->DatabaseDiagnosticRecords;
    i =0; 

    if (info->Diagnostics != NULL) {
      showDiags(info->Diagnostics);
      return NULL;
    }

    /* WAIS will return 1 "result" (the catalogue) if there are no
       results to the query.  Check for this and return null
       in this case - score 0 in the one document returned
       appears to indicate this*/
    if (response->NumberOfRecordsReturned == 1 &&
	info->DocHeaders[0]->Score == 0) {
      
      return NULL;
    }


    if ( info->DocHeaders != 0 ) {
      k =0;
      while (info->DocHeaders[k] != 0 ) {
	DocID *docid = docIDFromAny(info->DocHeaders[k]->DocumentID);
	char *docidstring = anyToString(GetLocalID(docid));
	i++;
	

	responses[k] = (char *)
	  s_malloc((size_t)RESP_BUF_LEN * sizeof(char));

	/* docidstring has been added to standard waissearch response. */
	sprintf(responses[k], "%ld %ld %ld '%s' '%s'", 
	       (info->DocHeaders[k]->Score),
	       info->DocHeaders[k]->Lines,
	       info->DocHeaders[k]->DocumentLength,
	       docidstring,
	       trim_junk(info->DocHeaders[k]->Headline));
	retstrs->body[k] = makeJavaString(responses[k], strlen(responses[k]));
	k++;
      }
    }
  }
  return(retval);
}

#define MAX_KEYWORDS_LENGTH 1000
#define MAX_SERVER_LENGTH 1000
#define MAX_DATABASE_LENGTH 1000
#define MAX_SERVICE_LENGTH 1000

/*
  This is the "method" called through the native call.
  */
HArrayOfString *
wais_WAISSearchSend_SearchWAIS(this_h, in_host, in_port, in_database, in_terms)
Hwais_WAISSearchSend *this_h;
Hjava_lang_String *in_host;
Hjava_lang_String *in_port;
Hjava_lang_String *in_database;
Hjava_lang_String  *in_terms;
		
{
  char userInfo[500];
  char* request_message = NULL; /* arbitrary message limit */
  char* response_message = NULL; /* arbitrary message limit */
  long request_buffer_length;	/* how of the request is left */
  SearchResponseAPDU  *query_response;
  SearchResponseAPDU  *retrieval_response;
  WAISSearchResponse  *query_info, *retrieval_info;
  char keywords[MAX_KEYWORDS_LENGTH + 1];
  char server_name[MAX_SERVER_LENGTH + 1];	
  char service[MAX_SERVICE_LENGTH + 1];
  char database[MAX_DATABASE_LENGTH + 1];
  char *next_argument, *command_name;
  long count, Max_Docs = 500, message_length = MAX_MESSAGE_LEN;
  FILE *connection;
  char *err_buf;
  char **resp_list;
  HArrayOfString *retval;

  database[0] = '\0';  /* null it out */
  service[0] = '\0';


  /* This is the replacement for the original get arguments code */
  strncpy(server_name, allocCString(in_host), MAX_SERVER_LENGTH);
  strncpy(service, allocCString(in_port), MAX_SERVICE_LENGTH);
  strncpy(database, allocCString(in_database), MAX_DATABASE_LENGTH);
  strncpy(keywords, allocCString(in_terms), MAX_KEYWORDS_LENGTH);

  if (server_name[0] == '\0' && service[0] == '\0')
    connection = NULL; /* do local searching */
  else /* remote search, fill in defaults if necessary */
    { if (server_name[0] == '\0')
      mygethostname(server_name,MAX_SERVER_LENGTH); /*default to local machine*/
    if (service[0] == '\0')
      strcpy(service, Z39_50_SERVICE); /* default */
    if ((connection = connect_to_server(server_name,atoi(service))) == NULL) 
      {	
	err_buf = (char *)
	  s_malloc((size_t)ERR_BUF_LEN * sizeof(char));
	sprintf (err_buf, "Error opening connection to %s via service %s.\n",
		 server_name, service);
	throw_e(err_buf);
	return NULL;
      }
    }

  /* init logging if we want to.  this is just for timing studies */
  read_environment_variables(server_name, service);

  request_message = (char*)s_malloc((size_t)message_length * sizeof(char));
  response_message = (char*)s_malloc((size_t)message_length * sizeof(char));

  {
    char hostname[80];
/*    char domainname[80]; */
    char *domainname;
    struct hostent *my_host_ent;

    mygethostname(hostname, 80);
/* Security patch from mike@tab00.larc.nasa.gov */
    if (((my_host_ent = gethostbyname(hostname)) == NULL) ||
	(domainname = (char *)index(my_host_ent->h_name, '.')) == NULL)
      domainname = "unknown";
    else
      domainname++;
/*    getdomainname(domainname, 80); */

    if((message_length = 
	init_connection(request_message, response_message,
			message_length,
			connection,
			userInfo)) <= 0) {
      err_buf = (char *)
	  s_malloc((size_t)ERR_BUF_LEN * sizeof(char));
      sprintf (err_buf, "Error opening connection to %s via service %s.\n",
	       server_name, service);
      throw_e(err_buf);
      return NULL;
    }
  }

    request_buffer_length = message_length; /* how of the request is left */
    if(NULL ==
	generate_search_apdu(request_message + HEADER_LENGTH, 
			     &request_buffer_length, 
			     keywords, database, NULL, Max_Docs)) {
      err_buf = (char *)
	  s_malloc((size_t)ERR_BUF_LEN * sizeof(char));

      sprintf(err_buf, "Error creating search APDU: request too large");
      throw_e(err_buf);
      return NULL;
    }

    if(0 ==
       interpret_message(request_message, 
			 message_length - request_buffer_length, 
			 response_message,
			 message_length,
			 connection,
			 false	/* true verbose */
			 )) { /* perhaps the server shut down on us, let's see: */
      if ( connection != NULL) {
	fclose(connection);
	if ((message_length = 
		  init_connection(request_message, response_message,
				  message_length,
				  connection,
				  userInfo)) <= 0)
	  {
	    err_buf = (char *)
	      s_malloc((size_t)ERR_BUF_LEN * sizeof(char));
	    sprintf (err_buf, "Error opening connection to %s via service %s.\n",
		     server_name, service);
	    throw_e(err_buf);
	    return NULL;
	  }
	if(0 ==
	   interpret_message(request_message, 
			     message_length - request_buffer_length, 
			     response_message,
			     message_length,
			     connection,
			     false /* true verbose */
			     )) {
	  throw_e("really couldn't deliver message");
	  return NULL;
	}
	  
	}
      else {
	throw_e("returned message too large");
	return NULL;
	  }
    }

    readSearchResponseAPDU(&query_response, response_message + HEADER_LENGTH);
    retval = display_search_response(query_response);
    query_info = (WAISSearchResponse *)query_response->DatabaseDiagnosticRecords;

    freeWAISSearchResponse(query_response->DatabaseDiagnosticRecords);         
    freeSearchResponseAPDU( query_response);

    close_connection(connection);


  s_free(request_message);
  s_free(response_message);

  return(retval);

}
